function AppView() {
    var that=this;
    $('input[name="callee"]').keypress(function(e){
        if (e.which === 13) {
            app.callee = e.currentTarget.value;
            if (that.isValidInput(app.callee)) {
                e.preventDefault();
                console.log("Callee set to " + app.callee);
                app.mediaEngine.startCall();
            }
            else {
                that.stateChange('INVALID');
            }
        }
    });

    $('#login').submit(function(e){
        e.preventDefault();
        app.storage.setItem('username', 
            e.currentTarget.elements.namedItem('username').value);
        app.serverConnection.login(
            e.currentTarget.elements.namedItem('password').value);
    });

    $('#testCall').click(function(e){
        app.callee = '+470047004700';
        app.mediaEngine.startCall();
        $('#testCall').hide();
    })

    $('#endCall').click(function(e){
        app.mediaEngine.endCall();
        that.stateChange('SHUTDOWN');
    })
}

AppView.prototype.isValidInput = function(callee) {
    // First, check if the input is a valid phone number
    var stripped_number = callee.replace(/[^0-9]+/g,'');

    // RFC 2822 email regex credit: 
    // http://www.hacksparrow.com/javascript-email-validation.html
    var email_regex = RegExp(['^(([^<>()[\\]\\\\.,;:\\s@\\"]',
        '+(\\.[^<>()[\\]\\\\.,;:\\s@\\"]+)*)|',
        '(\\".+\\"))@((\\[[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]',
        '{1,3}\\])|(([a-zA-Z\\-0-9]+\\.)+[a-zA-Z]{2,}))$'].join(''),'i');


    if ((stripped_number.length === 10 && 
        stripped_number.split(47).reverse()[0].length === 8) || 
        (stripped_number.length === 8)) {
        return true;
    } 
    // Continue to check if the input is an RFC 2822 standardized email
    else if (email_regex.test(callee)) {
        return true;
    }
    return false;
}

AppView.prototype.hideLoginForm = function() {
    $('#login').hide();
}

AppView.prototype.stateChange = function(state) {
    switch (state) {
        case 'RINGING': 
            $('input[name="callee"]').prop('disabled', true);
            $('#endCall').show();
            $('#call_indicator').text('Ringing...');
            break;

        case 'ANSWER':
            $('input[name="callee"]').prop('disabled', true);
            $('#endCall').show();
            $('#call_indicator').text('Answered!');
            break;

        case 'PROCESSING':
            $('input[name="callee"]').prop('disabled', true);
            $('#endCall').show();
            $('#call_indicator').text('Connecting...');
            break;

        case 'INVALID':
            $('#call_indicator').text('Invalid number or username');
            break;
            
        case 'SHUTDOWN':
            $('#call_indicator').text('Call a number or user');
            $('input[name="callee"]').prop('disabled', false);
            $('#endCall').hide();
            $('#testCall').show();
            break;
    }
    
}

AppView.prototype.playStream = function(stream) {
    var phone;
    phone = document.getElementById('phone');
    phone.src = window.webkitURL.createObjectURL(stream);
    phone.play();
}

AppView.prototype.stopStream = function(stream) {
    var phone;
    phone = document.getElementById('phone');
    phone.pause();
}