function CommandHandler(serverConnection) {
  this.serverConnection = serverConnection;
}

CommandHandler.prototype.sendSubscriptionCommand = function(callback) {
  var message = {
    subscriptionInformation: {
      subscribeToInstantMessagesOfType: 'application/sdp'
    }
  }
  this.serverConnection.sendMessage('SubscriptionCommand', message);
}

CommandHandler.prototype.sendServiceRequestCommand = function(callback) {
  var message = {
    serviceId: 'talkplus'
  }

  this.serverConnection.sendMessage('ServiceRequestCommand', message);
}

CommandHandler.prototype.sendClientRegistrationCommand = function(callback) {
  var message;
  if (app.storage.getItem('clientId') !== null) {
    this.sendAccountLoginCommand();
    return;
  }
  message = {
    clientInformation: {
      imsi:"talkplusweb",
      imei:"talkplusweb"
    }
    
  };
  this.serverConnection.sendMessage('ClientRegistrationCommand', message);
}


CommandHandler.prototype.handleClientRegistrationResponse = function(data) {
  var clientId = 
  data['com.telenor.sw.footee.common.th.ClientRegistrationResponse'].clientId
    app.storage.setItem('clientId', clientId);
    this.sendAccountLoginCommand();

  }

  CommandHandler.prototype.sendAuthenticateSessionCommand = 
  function(callback, message) {
    var message = {
      authenticateSessionInformation: {
        userId: app.storage.getItem('userId'),
        clientId: app.storage.getItem('clientId'),
        sessionKey: app.storage.getItem('sessionKey')
      }
    }
    this.serverConnection.sendMessage(
      'AuthenticateSessionCommand', message);
  }

  CommandHandler.prototype.handleAuthenticateSessionResponse = 
  function(data) {
    var response = 
    data['com.telenor.sw.footee.common.th.AuthenticateSessionResponse'];
    this.serverConnection.setAuthenticated(response.authenticated);
    if (response.authenticated) {
      console.log("Logged in!");
    }
    else {
      console.log("Could not authenticate with token, resetting");
      app.storage.setItem('userId', ''),
      app.storage.setItem('clientId', ''),
      app.storage.setItem('sessionKey', '')
    }
  }

  CommandHandler.prototype.sendAccountLoginCommand = 
  function(callback, password) {
    var message;

    if (app.storage.getItem('sessionKey')) {
      this.sendAuthenticateSessionCommand()
      return;
    }
    message = {
      accountLoginInformation: {
        userName: app.storage.getItem('username'),
        password: password,
        clientId: app.storage.getItem('clientId')
      }
    }
    this.serverConnection.sendMessage('AccountLoginCommand', message);
  }

  CommandHandler.prototype.handleAccountLoginResponse = function(data) {
    var response = data['com.telenor.sw.footee.common.th.AccountLoginResponse'];

    this.serverConnection.setAuthenticated(response.loggedIn);
    
    app.storage.setItem('sessionKey', response.sessionKey);
    app.storage.setItem('userId', response.userId);
  }

  CommandHandler.prototype.sendInstantMessageCommand = 
  function(callback, message) {
    var message = { 
      instantMessage: {
        contentBody: message.contentBody,
        recipientUserName: message.recipientUserName,
        contentType: message.contentType,
        originatingUserName: message.originatingUserName
      }
    }
    this.serverConnection.sendMessage('SendInstantMessageCommand', message);
  }

  CommandHandler.prototype.handleNewInstantMessage = function(data) {
    var instantMessage;

    instantMessage = 
    data['com.telenor.sw.footee.common.th.NewInstantMessage']
    ['instantMessage'];

    app.mediaEngine.handleSignalingInstantMessage(instantMessage);

  }

  CommandHandler.prototype.sendMideeAccessRequestCommand = 
  function(callback) {
    var message = {
      request: {
        token: 'abc'
      }
    }
    this.serverConnection.sendMessage('MideeAccessRequestCommand', message);
  }   

  CommandHandler.prototype.handleMideeAccessResponse = function(data) {
    var rtc, data;
    data = data['com.telenor.sw.footee.common.th.MideeAccessResponse'];
    app.mediaEngine.createPeerConnection(data.rtp)
  }
