// Find the browser specific API
RTCPeerConnection = window.webkitRTCPeerConnection || 
  window.mozRTCPeerConnection || window.RTCPeerConnection;

navigator.getUserMedia = navigator.webkitGetUserMedia || 
  navigator.mozGetUserMedia || navigator.getUserMedia;

RTCSessionDescription = window.mozRTCSessionDescription || 
  window.RTCSessionDescription

function MediaEngine() {
  this.peerConnection = null;
  this.localMediaStream = null;
  this.callWaitingForIce = false;
  this.sessionId = 1460;
  this.sequence = 1;
  this.sessionToken = null;
  this.turnInfo = null;
};

MediaEngine.prototype.createPeerConnection = function(turnInfo) {
  var that=this, config;
  if (turnInfo !== undefined) {
    this.turnInfo = turnInfo;
  }
  config = {
    iceServers:
    [
    {
      url:'turn:'+that.turnInfo.server+':6768', 
      credential: that.turnInfo.password,
      username: that.turnInfo.user
    }]
  }
  console.log(config);
  try {
    // As of May 11th, Chrome Stable does not support the username field
    // To leverage this, we fall back to the old format if the new fails
    that.peerConnection = new RTCPeerConnection(config);
  }
  catch (e) {
    config.iceServers[0] = {
      url:'turn:'+that.turnInfo.user+'@'+that.turnInfo.server+':6768', 
      credential: that.turnInfo.password
    }
    that.peerConnection = new RTCPeerConnection(config);
  }


  that.peerConnection.onicecandidate = function(e) {
    // If the last candidate is null, 
    // we know the ICE gathering has finished
    if (e.candidate === null) {
      console.log("ICE gathering complete")
      if (that.callWaitingForIce) {
        that.startCall();        
      }
      return
    }
    console.log("icecandidate: " + e.candidate.candidate);
  }

  that.peerConnection.oniceconnectionstatechange = function(e) {
    if (that.peerConnection.iceConnectionState === "connected") {
      console.log("Connected to Talk+ relay!");
    }
  }

  that.peerConnection.onaddstream = function(e) {
    console.log("Remote stream added")
    app.appView.playStream(e.stream);
  }

  if (that.localMediaStream === null) {
    that.requestMicrophone();
  }
  else {
    that.createOffer();
  }
}

MediaEngine.prototype.startCall = function() {
  var that = this, contentBody, message;
  app.appView.stateChange('PROCESSING');

// Got all ICE candidates, let's send the offer
if (that.peerConnection.iceGatheringState === "complete") {
  this.callWaitingForIce = false;
  console.log("Sending offer");
  message = new SignalingMessage();
    contentBody = {
      messageType: "OFFER",
      sdp: that.peerConnection.localDescription.sdp,
      offererSessionId: ++that.sessionId,
      answererSessionId: '',
      seq: that.sequence
    }

    message.contentBody = window.btoa(JSON.stringify(contentBody));
    app.serverConnection.sendSignalingMessage(message);
    // console.log(contentBody);
  }
  else {
    console.log("Waiting for more ICE candidates")
    this.callWaitingForIce = true;
  }
};

MediaEngine.prototype.endCall = function() {
  var message, contentBody;
  console.log("Ending call");
  app.appView.stopStream();
  this.peerConnection.close();
  this.createPeerConnection();

  message = new SignalingMessage();
  contentBody = {
    messageType: "SHUTDOWN",
  }
  // If the call has started, supply the call session token
  if (this.sessionToken !== null) {
    contentBody.sessionToken = this.sessionToken;
  } 
  // If the call has not started, supply the offerer session ID
  else {
    contentBody.offererSessionId = this.sessionId;
  }

  message.contentBody = window.btoa(JSON.stringify(contentBody));
  app.serverConnection.sendSignalingMessage(message);
  this.sessionToken = null;
}

MediaEngine.prototype.createOffer = function() {
  var that=this;
  console.log("Creating offer");

  this.peerConnection.addStream(this.localMediaStream);
  this.peerConnection.createOffer(function(sessionDescription) {
      console.log(sessionDescription);
      that.peerConnection.setLocalDescription(sessionDescription);
    }, function(e){}
  );
}

MediaEngine.prototype.handleSignalingInstantMessage = function(message) {
  var contentBody, sessionDescription;
  var that = this;
  contentBody = JSON.parse(window.atob(message.contentBody));
  console.log("Got " + contentBody.messageType)

  // Update the UI to reflect  the current state
  app.appView.stateChange(contentBody.messageType);

  switch (contentBody.messageType) {
    case 'RINGING': {
      break;
    }
    case "ANSWER": {
      console.log(contentBody);
      sessionDescription = new RTCSessionDescription({
        type: 'answer',
        sdp: contentBody.sdp
      });

      that.peerConnection.setRemoteDescription(sessionDescription, 
        function(){
          console.log("Remote description set!");
          that.sessionToken = contentBody.setSessionToken;
        }, function(e){ 
          console.log("Remote description could not be set: " + e)
        }
        );

      break;
    }
    case "SHUTDOWN": {
          app.appView.stopStream();
          this.peerConnection.close();
          this.createPeerConnection();
          break;
        }
        default: {
          console.log("Unrecognized command: " + contentBody.messageType);
        }
      }
    };

    MediaEngine.prototype.requestMicrophone = function() {
      var that = this;

  // Request access to the microphone
  navigator.getUserMedia({audio: true}, function(localMediaStream) {
    that.localMediaStream = localMediaStream;
    that.createOffer();
  }, function(error){console.log(error);});
}
