function ServerConnection() {
    var that = this;

    this.authenticated = false;
    this.ws = null;
    this.commandHandler = new CommandHandler(this);

    this.ping = function() {
        setTimeout(function() {
            that.ws.send("\0");
            that.ping();
        }, 30*1000);
    }
};

ServerConnection.prototype.connect = function() {
    var that = this;
    this.ws = new WebSocket('wss://edgee-ws-api-staging.comoyo.com:443');

    this.ws.onclose = function() {
        console.log("Server connection closed");
    };
    this.ws.onopen = function() {
        var timeout;
        console.log("Server connection opened.");
        that.commandHandler.sendClientRegistrationCommand(that.sendMessage);
        that.ping();

    };
    this.ws.onmessage = function(message) {
        console.log("Server message received");
        // Handle incoming commands and relay the data
        var command, data, commandSuffix;

        if (message.data === "") {
            console.log("Received ping");
            return;
        }

        data = JSON.parse(message.data.substring(0, message.data.length - 1));
        commandSuffix = Object.keys(data)[0].split('.').reverse()[0];
        command = "handle" + commandSuffix;

        // Call the handle function for the incoming command, if it exists
        if (typeof(that.commandHandler[command]) !== "undefined" || 
            that.commandHandler.hasOwnProperty(command)) {
            that.commandHandler[command](data);
        }
    }  
};

ServerConnection.prototype.close = function() {
    console.log("Closing edgee connection");
    if (typeof(this.ws) !== "undefined" || ws.hasOwnProperty('close')) {
        this.ws.close();
    }
};

ServerConnection.prototype.sendMessage = function(command, message) {
    var fullCommand, data;

    fullCommand = "com.telenor.sw.adaptee.th." + command;
    data = {}
    data[fullCommand] = message;
    this.ws.send(JSON.stringify(data)); 
};

ServerConnection.prototype.readyState = function() {
    return this.ws.readyState;
};

ServerConnection.prototype.login = function(password) {
    this.commandHandler.sendAccountLoginCommand(null, password);
};

ServerConnection.prototype.sendSignalingMessage = function(message) {
    this.commandHandler.sendInstantMessageCommand(null, message);
};

ServerConnection.prototype.setAuthenticated = function(authenticated) {
    if (authenticated === true) {
        this.commandHandler.sendSubscriptionCommand();
        this.commandHandler.sendMideeAccessRequestCommand();

        app.appView.hideLoginForm();
    }
};

function SignalingMessage() {
    this.contentBody = null;
    this.recipientUserName = app.callee;
    this.originatingUserName = app.storage.getItem('username');
    this.contentType = "application/sdp";
}
